/*  Copyright (C) 2008--2009 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#ifndef VISUALID_LINE_H
#define VISUALID_LINE_H

struct line_generator {
  VisualID_Glyph base;

  double len; /* length of the line */
  int n; /* number of C3 children along the line */

  VisualID_Glyph *ec1; /* end-child 1 */
  double c1_scale;

  VisualID_Glyph *ec2; /* end-child 2 */
  double c2_scale;

  VisualID_Glyph *c3; /* child 3 (along the line) */
  double c3_scale;

  int c3_doublesided; /* bool: child-3 is drawn mirrored on both sides */
  double c3_angle; /* angle of child-3 relative to line normal */

  int c3_taper; /* bool: reduce or increase scale of child-3 along the line */
  int c3_increase; /* bool: increase (rather than decrease) child-3 scale */
};

VisualID_Glyph *gen_line (VisualID_Glyph *parent, const char *attachment);

#define VISUALID_TYPE_LINE            (visualid_line_get_type ())
#define VISUALID_LINE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), VISUALID_TYPE_LINE, VisualID_Line))
#define VISUALID_IS_LINE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VISUALID_TYPE_LINE))
#define VISUALID_LINE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), VISUALID_TYPE_LINE, VisualID_Line))
#define vISUALID_IS_LINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VISUALID_TYPE_LINE))
#define VISUALID_LINE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), VISUALID_TYPE_LINE, VisualID_Line))

typedef struct line_generator VisualID_Line;
typedef struct _VisualID_LineClass VisualID_LineClass;

GType visualid_line_get_type (void);

#endif
