/*  Copyright (C) 2008--2009 Joshua Judson Rosen <rozzin@geekspace.com>.

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    version 3 as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING. If not, see
    <http://www.gnu.org/licenses/> or write to:

        The Free Software Foundation, inc.
        51 Franklin Street, Fifth Floor
        Boston, MA 02110-1301
        USA
*/

#ifndef VISUALID_RADIAL_H
#define VISUALID_RADIAL_H

struct radial_generator {
  VisualID_Glyph base;

  int n; /* the number of vertices in the n-gon */

  VisualID_Glyph *vertex_child; /* markers at the vertices, if they will
                                 be expanded */
  float vc_scale; /* scale-reduction factor for vertex children */

  VisualID_Glyph *mouth_child; /* markers in the centre, if they will be
                                  expanded */
  float mc_scale; /* scale-reduction factor for the mouth child */
  float mouth_org; /* an offset of the centre child down from the
                      centre of the n-gon */

  VisualID_Glyph *eye_child; /* places two additional children that
                              might be interpreted as "eyes"? */
  float ec_scale; /* their scale reduction */
  float eye_sep; /* the horizontal separation between these */
  float eye_org; /* the offset of the "eyes" up from the center */

  float angle_cutoff; /* vertex children appear only within this
                         angular range (in radians) from the top of
                         the n-gon */
};

VisualID_Glyph *gen_radial (VisualID_Glyph *parent, const char *attachment);

#define VISUALID_TYPE_RADIAL            (visualid_radial_get_type ())
#define VISUALID_RADIAL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), VISUALID_TYPE_RADIAL, VisualID_Radial))
#define VISUALID_IS_RADIAL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VISUALID_TYPE_RADIAL))
#define VISUALID_RADIAL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), VISUALID_TYPE_RADIAL, VisualID_RadialClass))
#define VISUALID_IS_RADIAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VISUALID_TYPE_RADIAL))
#define VISUALID_RADIAL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), VISUALID_TYPE_RADIAL, VisualID_RadialClass))

typedef struct radial_generator VisualID_Radial;
typedef struct _VisualID_RadialClass VisualID_RadialClass;

GType visualid_radial_get_type (void);

#endif
